<?php
/*
	class.pwdhash.php v1.4.4 - A password hash and compare class.
	(c) 2010-2015 Andreas Tunberg <http://tunisempire.com>
	MIT-style license.
	Date: 2015-02-05
*/
class pwdhash
{
	private $pass,$salt,$hash,$hexsalt,$hexhash,$hex,$utf8,$rounds,$passspec;
	private $passlen=10,$passstr=3;
	private $algo='sha256';
	private $newinput=TRUE;
	public function __construct($pass='',$salt=NULL,$hex=TRUE,$utf8=FALSE,$rounds=0)
	{		
		$this->utf8=is_bool($utf8)?$utf8:FALSE;
		$this->hex=is_bool($hex)?$hex:TRUE;
		$this->pass=$this->utf8?utf8_decode($pass):$pass;
		$this->salt=(!$salt||is_int($salt))?$this->make_salt($salt):($this->utf8?utf8_decode($salt):$salt);
		$this->rounds=(int)$rounds?$rounds:256;
	}
	public function set_algorithm($algo='')
	{
		if(!in_array($algo,hash_algos()))return FALSE;
		$this->algo=$algo;
		return TRUE;
	}
	public function new_password($pass='',$length=0,$special_chars='')
	{
		if(!$pass)return FALSE;
		if($pass===TRUE)
		{
			if((int)$length)$this->passlen=$length;
			if($special_chars)$this->passspec=$this->utf8?utf8_decode($special_chars):$special_chars;
			$this->random_password();
			return $this->utf8?utf8_encode($this->pass):$this->pass;
		}
		$this->pass=$this->utf8?utf8_decode($pass):$pass;
		$this->newinput=TRUE;
		return TRUE;
	}
	public function new_salt($salt=NULL)
	{
		$this->salt=(!$salt||is_int($salt))?$this->make_salt($salt):($this->utf8?utf8_decode($salt):$salt);
		$this->newinput=TRUE;
		$this->hexsalt=NULL;
		return TRUE;
	}
	public function salt($hex=NULL)
	{
		$hex=is_bool($hex)?$hex:$this->hex;
		if($hex&&!$this->hexsalt)$this->hexsalt=$this->hex($this->salt);
		return $hex?$this->hexsalt:($this->utf8?utf8_encode($this->salt):$this->salt);

	}
	public function hash($hex=NULL)
	{
		$hex=is_bool($hex)?$hex:$this->hex;
		if($this->newinput&&!$this->make_hash())return FALSE;
		if($hex&&!$this->hexhash)$this->hexhash=$this->hex($this->hash);
		return $hex?$this->hexhash:$this->hash;
	}
	public function hashkey($hex=NULL)
	{
		return($hash=$this->hash($hex))?$this->get_salt($hex).$hash:FALSE;
	}
	public function compare($key='',$hex=NULL)
	{
		if(!$key)return FALSE;
		$hex=is_bool($hex)?$hex:$this->hex;
		if($hex)$key=$this->dehex($key);
		$salt=substr($key,0,strlen($this->salt));
		$hash=substr($key,strlen($this->salt));
		return $this->rounds($salt)==$hash?TRUE:FALSE;
	}
	private function make_salt($length=32)
	{
		if(!(int)$length)$length=32;
		$salt='';
		for($i=0;$i<$length;++$i)
		{
			$salt.=chr(rand(0,255));
		}
		return $salt;
	}
	private function hex($string)
	{
		$hex='';
		$strlen=strlen($string);
		for($i=0;$i<$strlen;++$i)
		{
			$hex.=bin2hex($string{$i});
		}
		return $hex;
	}
	private function dehex($hex)
	{
		$string='';
		$strlen=strlen($hex);
		for($i=0;$i<$strlen;$i=$i+2)
		{
			$string.=chr(hexdec(substr($hex,$i,2)));
		}
		return $string;
	}
	private function make_hash()
	{
		if(!$this->hash=$this->rounds($this->salt))return FALSE;
		$this->newinput=FALSE;
		$this->hexhash=NULL;
		return TRUE;
	}
	private function rounds($salt)
	{
		$hash=$this->pass;
		for($i=0;$i<$this->rounds;++$i)
		{
			if(!$hash=hash($this->algo,$hash.$salt,TRUE))return FALSE;
		}
		return $hash;
	}
	private function random_password()
	{
		# No 'lI1,oO0' they can look a like.
		$chars = 'abcdefghijkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ23456789';
		if($this->passspec)$chars.=$this->passspec;
		$size = strlen($chars)-1;
		for($i=0;$i<$this->passlen;++$i)
		{
			$str.=$chars[rand(0,$size)];
		}
		$this->newinput=TRUE;
		$this->pass=$str;
	}
	private function get_salt($hex=NULL)
	{
		$hex=is_bool($hex)?$hex:$this->hex;
		if($hex&&!$this->hexsalt)$this->hexsalt=$this->hex($this->salt);
		return $hex?$this->hexsalt:$this->salt;
	}
}
?>